-module(listener).
-export([start/1, loop/3]).


start(Port) ->
	spawn(fun() -> loop(Port, nothing, nothing) end),
	ok.


loop(Port, MyQueue, Neighbours) ->
	io:format("Listening on port ~p~n", [Port]),
	{ok, LSock} = gen_tcp:listen(Port, [binary, {packet, 0}, {active, false}]),
	io:format("Listening on sock ~p~n", [LSock]),
	spawn(fun() -> accept(LSock, MyQueue, Neighbours) end),
    timer:sleep(infinity).


accept(LSock, MyQueue, Neighbours) ->
    io:format("Accepting on sock ~p~n", [LSock]),
    case gen_tcp:accept(LSock) of
        {ok, Sock} ->
            io:format("Accepted, sock: ~p~n", [Sock]),
            recv_msg(Sock, MyQueue, Neighbours, []),
            accept(LSock, MyQueue, Neighbours),
            ok;
        Other ->
            io:format("accept Something happened: ~p~n", [Other])
    end.


%% Couldn't find right key on keyboard to send with telnet, so ` is EOF too 
has_eol(RawData) -> 
    io:format("has_eol: ~p :~n", [RawData]),
    case binary:match(RawData, [<<$\r, $\n>>, <<$\n>>, <<"`">>]) of 
        nomatch -> 
            io:format("has_eol: ~p : false~n", [RawData]),
            false;
        _ -> 
            io:format("has_eol: ~p : true~n", [RawData]),
            true
    end.


recv_msg(Sock, MyQueue, Neighbours, AccReceived) ->
	io:format("Receiving message on sock ~p~n", [Sock]),
	case gen_tcp:recv(Sock, 0) of
        {ok, D} ->
        	io:format("received packet: ~p~n", [D]),
            case has_eol(D) of
                true -> 
                    Result = process_data(list_to_binary(lists:reverse([D | AccReceived])), MyQueue, Neighbours),
                    case Result of 
                        ok -> 
                            gen_tcp:send(Sock, "OK\n");
                        {error, Reason} -> 
                            RawMessage = list_to_binary(["ERROR ", Reason, "\n"]),
                            gen_tcp:send(Sock, RawMessage)
                    end,
                    io:format("Closing sock ~p~n", [Sock]),
                    gen_tcp:close(Sock),
                    ok;  
                false ->
                    recv_msg(Sock, MyQueue, Neighbours, [D | AccReceived]),
                    ok    
            end;
        {error, closed} ->
            io:format("socket closed!~n"),
            ok;
        Other ->
        	io:format("recv_msg Something happened: ~p~n", [Other])
    end.


process_data(Msg, MyQueue, Neighbours) ->
    io:format("Process data: ~p~n", [Msg]),
    case binary:match(Msg, <<"SEND">>) of
        nomatch ->
            case binary:match(Msg, <<"SUBSCRIBE">>) of
                nomatch ->
                    {error, "Unexpected message type"};
                {Start, Len} ->
                    Body = binary:part(Msg, {Start + Len + 1, size(Msg) - Len - 2}),
                    process_sub(Body)
            end;
        {Start, Len} ->
            Body = binary:part(Msg, {Start + Len + 1, size(Msg) - Len - 2}),
            process_send(Body, MyQueue, Neighbours)
    end.


process_send(MsgBody, MyQueue, Neighbours) ->
    try 
        io:format("process_send: ~p~n", [MsgBody]),
        [{Start, Len} | _] = binary:matches(MsgBody, <<" ">>),
        MessageTag = binary:part(MsgBody, {0, Start}),
        Message = binary:part(MsgBody, {Start + 1, size(MsgBody) - Start - 1}),
        io:format("MessageTag: ~p, Message: ~p~n", [MessageTag, Message]),
        queues:put_tag(MessageTag, Message)
    catch
        E -> {error, E}
    end.

binary_to_integer(B) -> list_to_integer(binary_to_list(B)).

process_sub(MsgBody) ->
    try 
        io:format("process_sub, ~p~n", [MsgBody]),
        [{Start, Len} | _] = binary:matches(MsgBody, <<" ">>),
        case catch(binary_to_integer(binary:part(MsgBody, {0, Start}))) of
            {'EXIT', Some} -> {error, Some};
            Port ->
                Pattern = binary:part(MsgBody, {Start + 1, size(MsgBody) - Start - 1}),
                io:format("Id: ~p, SubPattern: ~p~n", [Port, Pattern]),
                queues:subscribe_tag(Pattern, queues:subscriber({127,0,0,1}, Port))
        end
    catch        
        E -> {error, E}
    end.
