-module(queues).
-export([init/0, subscribe_tag/2, put_tag/2, subscriber/2]).

-define(TAGS, tags_table).

-record(subr, {
	host="localhost",
	port}).


subscriber(Host, Port) ->
	#subr{host = Host, port = Port}.


init() ->
	ets:new(?TAGS, [named_table, bag]).


subscribe_tag(MessageTag, Subscriber) ->
	case ets:info(?TAGS) of
		undefined -> init();
		_ -> ?TAGS
	end,
	case ets:lookup(?TAGS, MessageTag) of
		[] -> 
			io:format("Creating queue for tag: ~p~n", [MessageTag]),
			Q = start_queue(),
			ets:insert(?TAGS, [{MessageTag, Q}]),
			subscribe_tag(MessageTag, Subscriber);
		Queues ->
			io:format("Already has queues: ~p for tag: ~p~n", [Queues, MessageTag]),
			subscribe_to_queues(Queues, MessageTag, Subscriber)
	end.


put_tag(MessageTag, Message) ->
	case get_queues(MessageTag) of 
		{ok, Qs} -> 
			io:format("Found queues: ~p for tag ~p~n", [Qs, MessageTag]),
			put_to_queues(Qs, MessageTag, Message);
		{error, Reason} -> 
			io:format("Found something else: ~p for tag ~p~n", [{error, Reason}, MessageTag]),
			{error, Reason}
	end.


get_queues(MessageTag) ->
	io:format("get queues for tag: ~p~n", [MessageTag]),
	case ets:lookup(?TAGS, MessageTag) of
		[] -> {error, <<"No queue for tag: ", MessageTag/binary>>};
		Qs -> {ok, Qs}
	end.


subscribe_to_queues([], _, _) ->
	ok;
subscribe_to_queues([{MessageTag, Pid} | Rest], MessageTag, S) ->
	io:format("subscribe_to_queues Subscr to queue: ~p subscriber: ~p", [{MessageTag, Pid}, S]),
	Pid ! S,
	subscribe_to_queues(Rest, MessageTag, S).


put_to_queues([], _, _) ->
	ok;
put_to_queues([{MessageTag, Pid} | Rest], MessageTag, Message) ->
	io:format("put_to_queues Put to queue: ~p message: ~p", [{MessageTag, Pid}, Message]),
	Pid ! {message, MessageTag, Message},
	put_to_queues(Rest, MessageTag, Message).


start_queue() ->
	Pid = spawn_link(fun() -> queue_loop([]) end),
	io:format("start_queue Started queue with Pid: ~p~n", [Pid]),
	Pid.


queue_loop(Subscribers) ->
	receive
		{message, MessageTag, Message} ->
			io:format("queue_loop Queue with pid: ~p for message tag: ~p got message: ~p~n", [MessageTag, self(), Message]),
			send_to_subscribers(Subscribers, {message, MessageTag, Message}),
			queue_loop(Subscribers);
		S = #subr{} ->
			io:format("queue_loop Queue with pid: ~p got subscriber: ~p~n", [self(), S]),
			queue_loop([S | Subscribers])
	end.


send_to_subscribers([], _) ->
	ok;
send_to_subscribers([#subr{host = Host, port = Port} | Ss], {message, MessageTag, Message}) ->
	RawMessage = list_to_binary([<<"MESSAGE ">>, MessageTag, " ", Message, "\n"]),
	io:format("send_to_subscribers Queue for message tag: ~p sends: ~p~n", [MessageTag, RawMessage]),
	{ok, Sock} = gen_tcp:connect(Host, Port, [binary, {packet, 0}]),
    ok = gen_tcp:send(Sock, RawMessage),
    ok = gen_tcp:close(Sock),
    send_to_subscribers(Ss, {message, MessageTag, Message}).
