-module(sub_client).
-export([subscribe_and_start/5]).


subscribe_and_start(Id, Tag, MyPort, QHost, QPort) ->
	spawn(fun() -> start(Id, Tag, MyPort, QHost, QPort) end).


start(Id, Tag, MyPort, QHost, QPort) ->
	spawn(fun() -> listen(MyPort) end),
    subscribe(Id, Tag, QHost, QPort),
    ok.

subscribe(Id, Tag, QHost, QPort) ->
	{ok, Sock} = gen_tcp:connect(QHost, QPort, [binary, {packet, 0}]),
    RawMessage = list_to_binary([<<"SUBSCRIBE ">>, Id, <<" ">>, Tag, <<"\n">>]),
    ok = gen_tcp:send(Sock, RawMessage),
    ok = gen_tcp:close(Sock).


listen(Port) ->
	io:format("sub_client Listening on port ~p~n", [Port]),
	{ok, LSock} = gen_tcp:listen(Port, [binary, {packet, 0}, {active, false}]),
	io:format("sub_client Listening on sock ~p~n", [LSock]),
	spawn(fun() -> accept(LSock) end),
    timer:sleep(infinity).


accept(LSock) ->
    io:format("sub_client Accepting on sock ~p~n", [LSock]),
    case gen_tcp:accept(LSock) of
        {ok, Sock} ->
            io:format("sub_client Accepted, sock: ~p~n", [Sock]),
            recv_msg(Sock, []),
            ok;
        Other ->
            io:format("sub_client accept Something happened: ~p~n", [Other])
    end.


has_eol(<<"`">>) -> 
    io:format("has_eol: ~p : true~n", [<<"`">>]),
    true;
has_eol(<<$\n>>) -> 
    io:format("has_eol: ~p : true~n", [<<$\n>>]),
    true;
has_eol(<<$\r, $\n>>) ->
    io:format("has_eol: ~p : true~n", [<<$\r, $\n>>]),
    true;
has_eol(_Other) -> 
    io:format("has_eol: ~p : false~n", [_Other]),
    false.


recv_msg(Sock, AccReceived) ->
	io:format("sub_client Receiving message on sock ~p~n", [Sock]),
	case gen_tcp:recv(Sock, 0) of
        {ok, D} ->
        	io:format("sub_client received packet: ~p~n", [D]),
            case has_eol(D) of
                true -> 
                    process_data(list_to_binary(lists:reverse([D | AccReceived])));
                false ->
                    recv_msg(Sock, [D | AccReceived]),
                    ok    
            end;
        {error, closed} ->
            io:format("sub_client socket closed!~n"),
            ok;
        Other ->
        	io:format("sub_client recv_msg Something happened: ~p~n", [Other])
    end.


process_data(Msg) ->
    io:format("sub_client Received: ~p~n", [Msg]).